@@ -31,6 +31,7 @@
 #include "hildon-desktop-notification-service.h"
 
 #include <string.h>
+#include <gtk/gtk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
 #define HILDON_DESKTOP_NOTIFICATION_MANAGER_GET_PRIVATE(object) \
@@ -38,6 +39,13 @@
 
 G_DEFINE_TYPE (HildonDesktopNotificationManager, hildon_desktop_notification_manager, GTK_TYPE_LIST_STORE);
 
+enum {
+  SIGNAL_NOTIFICATION_CLOSED,
+  N_SIGNALS
+};
+
+static gint signals[N_SIGNALS];  
+
 #define HILDON_DESKTOP_NOTIFICATION_MANAGER_DBUS_NAME  "org.freedesktop.Notifications" 
 #define HILDON_DESKTOP_NOTIFICATION_MANAGER_DBUS_PATH  "/org/freedesktop/Notifications"
 
@@ -123,10 +131,21 @@ hildon_desktop_notification_manager_init (HildonDesktopNotificationManager *nm)
 static void
 hildon_desktop_notification_manager_class_init (HildonDesktopNotificationManagerClass *class)
 {
+  GObjectClass *g_object_class = (GObjectClass *) class;
+
+  signals[SIGNAL_NOTIFICATION_CLOSED] =
+        g_signal_new ("notification-closed",
+                      G_OBJECT_CLASS_TYPE (g_object_class),
+                      G_SIGNAL_RUN_FIRST,
+		      G_STRUCT_OFFSET (HildonDesktopNotificationManagerClass, notification_closed),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__INT,
+                      G_TYPE_NONE, 1, G_TYPE_INT);
+
   g_type_class_add_private (class, sizeof (HildonDesktopNotificationManagerPrivate));
 }
 
-static gboolean 
+gboolean 
 hildon_desktop_notification_manager_find_by_id (HildonDesktopNotificationManager *nm,
 						guint id, 
 					        GtkTreeIter *return_iter)
@@ -156,25 +175,12 @@ hildon_desktop_notification_manager_find_by_id (HildonDesktopNotificationManager
 
 static DBusMessage *
 hildon_desktop_notification_manager_create_signal (HildonDesktopNotificationManager *nm, 
-		                                   guint                             id, 
-						   const char                       *signal_name)
+		                                   guint id,
+						   const gchar *dest, 
+						   const gchar *signal_name)
 {
   DBusMessage *message;
-  GtkTreeIter iter;
-  gchar *dest;
-
-  if (hildon_desktop_notification_manager_find_by_id (nm, id, &iter))
-  {
-    gtk_tree_model_get (GTK_TREE_MODEL (nm),
-		        &iter,
-			HD_NM_COL_SENDER, &dest,
-			-1);
-  }
-  else
-  {
-    return NULL;
-  }
-
+  
   g_assert(dest != NULL);
 
   message = dbus_message_new_signal ("/org/freedesktop/Notifications",
@@ -187,24 +193,20 @@ hildon_desktop_notification_manager_create_signal (HildonDesktopNotificationMana
                             DBUS_TYPE_UINT32, &id,
                             DBUS_TYPE_INVALID);
 
-  g_free (dest);
-  
   return message;
 }
 
 static void
 hildon_desktop_notification_manager_notification_closed (HildonDesktopNotificationManager *nm,
-		                                         GtkTreeIter *iter)
+		                                         guint id,
+							 const gchar *dest)
 {
   DBusMessage *message;
-  gint id;
   
-  gtk_tree_model_get (GTK_TREE_MODEL (nm), 
-                      iter,
-                      HD_NM_COL_ID, &id,
-                      -1);
-
-  message = hildon_desktop_notification_manager_create_signal (nm, id, "NotificationClosed");
+  message = hildon_desktop_notification_manager_create_signal (nm, 
+		  					       id, 
+							       dest, 
+							       "NotificationClosed");
 
   if (message == NULL) return;
 
@@ -213,21 +215,38 @@ hildon_desktop_notification_manager_notification_closed (HildonDesktopNotificati
 			NULL);
 
   dbus_message_unref (message);
+
+  g_signal_emit (nm, signals[SIGNAL_NOTIFICATION_CLOSED], 0, id);
 }
 
 static gboolean 
-hildon_desktop_notification_manager_timeout (GtkTreeIter *iter)
+hildon_desktop_notification_manager_timeout (gint id)
 {
   GtkListStore *nm = 
-     hildon_desktop_notification_manager_get_singleton ();	
+     hildon_desktop_notification_manager_get_singleton ();
+  GtkTreeIter iter;
+  gchar *dest;
+  
+  if (!hildon_desktop_notification_manager_find_by_id (HILDON_DESKTOP_NOTIFICATION_MANAGER (nm), 
+			  			       id, 
+						       &iter))
+  {
+    return FALSE;
+  }
 
+  gtk_tree_model_get (GTK_TREE_MODEL (nm),
+      	        &iter,
+      		HD_NM_COL_SENDER, &dest,
+      		-1);
+  
   /* Notify the client */
   hildon_desktop_notification_manager_notification_closed (HILDON_DESKTOP_NOTIFICATION_MANAGER (nm), 
-		  					   iter);
+		  					   id,
+							   dest);
  
-  gtk_list_store_remove (nm, iter);
+  gtk_list_store_remove (nm, &iter);
 
-  g_free (iter);
+  g_free (dest);
 
   return FALSE;
 }
@@ -274,11 +293,13 @@ hildon_desktop_notification_manager_notify (HildonDesktopNotificationManager *nm
                                             gint                   timeout, 
                                             DBusGMethodInvocation *context)
 {
-  GtkTreeIter iter,*iter_timeout;
+  GtkTreeIter iter;
   GError *error = NULL;
   GdkPixbuf *pixbuf = NULL;
   GtkIconTheme *icon_theme;
   GHashTable *hints_copy;
+  gchar **actions_copy;
+  gboolean valid_actions = TRUE;
   gint i;
   
   if (!g_str_equal (icon, ""))
@@ -324,9 +345,9 @@ hildon_desktop_notification_manager_notify (HildonDesktopNotificationManager *nm
       if (nm->priv->current_id == G_MAXUINT)
         nm->priv->current_id = 0;
     }
-
+    
     /* Test if we have a valid list of actions */
-    for (i = 0; actions[i] != NULL; i += 2)
+    for (i = 0; actions && actions[i] != NULL; i += 2)
     {
       gchar *label = actions[i + 1];
       
@@ -334,10 +355,21 @@ hildon_desktop_notification_manager_notify (HildonDesktopNotificationManager *nm
       {
         g_warning ("Invalid action list: no label provided for action %s", actions[i]);
 
-        break;
+	valid_actions = FALSE;
+
+	break;
       }
     }
 
+    if (valid_actions)
+    {
+      actions_copy = g_strdupv (actions);
+    }
+    else
+    {
+      actions_copy = NULL;
+    }
+	      
     hints_copy = g_hash_table_new_full (g_str_hash, 
 	  		                g_str_equal,
 			                (GDestroyNotify) g_free,
@@ -353,7 +385,7 @@ hildon_desktop_notification_manager_notify (HildonDesktopNotificationManager *nm
 		        HD_NM_COL_ICON, pixbuf,
 		        HD_NM_COL_SUMMARY, summary,
 		        HD_NM_COL_BODY, body,
-		        HD_NM_COL_ACTIONS, g_strdupv (actions),
+		        HD_NM_COL_ACTIONS, actions_copy,
 		        HD_NM_COL_HINTS, hints_copy,
 		        HD_NM_COL_TIMEOUT, timeout,
 			HD_NM_COL_REMOVABLE, TRUE,
@@ -374,13 +406,9 @@ hildon_desktop_notification_manager_notify (HildonDesktopNotificationManager *nm
 
   if (timeout > 0)
   {
-    iter_timeout = g_new0 (GtkTreeIter, 1);
-
-    *iter_timeout = iter;  
-    
     g_timeout_add (timeout,
 		   (GSourceFunc) hildon_desktop_notification_manager_timeout,
-		   iter_timeout);
+		   GINT_TO_POINTER (id));
   }
 		      
   dbus_g_method_return (context, id);
@@ -388,6 +416,86 @@ hildon_desktop_notification_manager_notify (HildonDesktopNotificationManager *nm
   return TRUE;
 }
 
+gboolean
+hildon_desktop_notification_manager_system_note_infoprint (HildonDesktopNotificationManager *nm,
+						           const gchar *message,
+                                                           DBusGMethodInvocation *context)
+{
+  GHashTable *hints;
+  GValue *hint;
+
+  hints = g_hash_table_new_full (g_str_hash, 
+        		         g_str_equal,
+      		                NULL,
+      		                (GDestroyNotify) hint_value_free);
+
+  hint = g_new0 (GValue, 1);
+  hint = g_value_init (hint, G_TYPE_STRING);
+  g_value_set_string (hint, "system.note.infoprint");
+
+  g_hash_table_insert (hints, "category", hint);
+
+  hildon_desktop_notification_manager_notify (nm,
+					      "hildon-desktop",
+		  			      0,
+					      "qgn_note_infoprint",
+		  			      "System Note Infoprint",
+					      message,
+					      NULL,
+					      hints,
+					      3000,
+					      context);
+
+  g_hash_table_destroy (hints);
+
+  return TRUE;
+}
+
+gboolean
+hildon_desktop_notification_manager_system_note_dialog (HildonDesktopNotificationManager *nm,
+						        const gchar *message,
+						        guint type,
+                                                        DBusGMethodInvocation *context)
+{
+  GHashTable *hints;
+  GValue *hint;
+
+  static const gchar *icon[4] = {
+      "qgn_note_gene_syswarning", /* OSSO_GN_WARNING */
+      "qgn_note_gene_syserror",   /* OSSO_GN_ERROR */
+      "qgn_note_info",            /* OSSO_GN_NOTICE */
+      "qgn_note_gene_wait"        /* OSSO_GN_WAIT */
+  };
+
+  g_debug ("ALOW ALOW ALOW ALOW");
+
+  hints = g_hash_table_new_full (g_str_hash, 
+        		         g_str_equal,
+      		                NULL,
+      		                (GDestroyNotify) hint_value_free);
+
+  hint = g_new0 (GValue, 1);
+  hint = g_value_init (hint, G_TYPE_STRING);
+  g_value_set_string (hint, "system.note.dialog");
+
+  g_hash_table_insert (hints, "category", hint);
+
+  hildon_desktop_notification_manager_notify (nm,
+					      "hildon-desktop",
+		  			      0,
+					      icon[type],
+		  			      "System Note Dialog",
+					      message,
+					      NULL,
+					      hints,
+					      3000,
+					      context);
+
+  g_hash_table_destroy (hints);
+  
+  return TRUE;
+}
+
 gboolean
 hildon_desktop_notification_manager_get_capabilities  (HildonDesktopNotificationManager *nm, gchar ***caps)
 {
@@ -422,13 +530,15 @@ hildon_desktop_notification_manager_close_notification (HildonDesktopNotificatio
                                                     	GError               **error)
 {
   GtkTreeIter iter;
+  gchar *dest;
   gboolean removable = TRUE;
-
+  
   if (hildon_desktop_notification_manager_find_by_id (nm, id, &iter))
   {
     gtk_tree_model_get (GTK_TREE_MODEL (nm),
 		        &iter,
 			HD_NM_COL_REMOVABLE, &removable,
+			HD_NM_COL_SENDER, &dest,
 			-1);
 
     /* libnotify call close_notification_handler when updating a row 
@@ -443,11 +553,13 @@ hildon_desktop_notification_manager_close_notification (HildonDesktopNotificatio
     else
     {
       /* Notify the client */
-      hildon_desktop_notification_manager_notification_closed (nm, &iter);
+      hildon_desktop_notification_manager_notification_closed (nm, id, dest);
 
       gtk_list_store_remove (GTK_LIST_STORE (nm), &iter);
     }
-      
+
+    g_free (dest);
+    
     return TRUE;    
   }
   else
@@ -530,7 +642,7 @@ hildon_desktop_notification_manager_message_from_desc (HildonDesktopNotification
   DBusMessage *message;
   gchar **message_elements;
   gint n_elements;
-  
+
   message_elements = g_strsplit (desc, " ", 5);
 
   n_elements = g_strv_length (message_elements);
@@ -587,49 +699,67 @@ hildon_desktop_notification_manager_call_action (HildonDesktopNotificationManage
 				                 const gchar                      *action_id)
 {
   DBusMessage *message = NULL;
+  GHashTable *hints = NULL; 
+  GtkTreeIter iter;
+  GValue *dbus_cb;
+  gchar *dest = NULL;
+  gchar *hint;
 
-  if (g_str_equal (action_id, "default")) 
+  g_return_if_fail (nm != NULL);
+  g_return_if_fail (HILDON_DESKTOP_IS_NOTIFICATION_MANAGER (nm));
+  
+  if (!hildon_desktop_notification_manager_find_by_id (nm, id, &iter))
   {
-    GHashTable *hints; 
-    GtkTreeIter iter;
-
-    if (hildon_desktop_notification_manager_find_by_id (nm, id, &iter))
-    {
-      GValue *dbus_cb;
-	    
-      gtk_tree_model_get (GTK_TREE_MODEL (nm),
-                          &iter,
-                          HD_NM_COL_HINTS, &hints,
-                          -1);
-
-      dbus_cb = (GValue *) g_hash_table_lookup (hints, "dbus-callback");
-
-      if (dbus_cb != NULL)
-      {
-        message = hildon_desktop_notification_manager_message_from_desc (nm, 
-			              (const gchar *) g_value_get_string (dbus_cb));
-      }
-
-      if (message != NULL)
-      {
-        dbus_connection_send (dbus_g_connection_get_connection (nm->priv->connection), 
-			      message, 
-			      NULL);
-      }
-    }
+    return;
   }
 
-  message = hildon_desktop_notification_manager_create_signal (nm, id, "ActionInvoked");
+  gtk_tree_model_get (GTK_TREE_MODEL (nm),
+                      &iter,
+                      HD_NM_COL_HINTS, &hints,
+                      HD_NM_COL_SENDER, &dest,
+                      -1);
 
-  g_assert (message != NULL);
+  if (hints != NULL)
+  {  
+    hint = g_strconcat ("dbus-callback-", action_id, NULL);
+    
+    dbus_cb = (GValue *) g_hash_table_lookup (hints, hint);
 
-  dbus_message_append_args (message,
-                            DBUS_TYPE_STRING, &action_id,
-                            DBUS_TYPE_INVALID);
+    if (dbus_cb != NULL)
+    {
+      message = hildon_desktop_notification_manager_message_from_desc (nm, 
+      		              (const gchar *) g_value_get_string (dbus_cb));
+    }
 
-  dbus_connection_send (dbus_g_connection_get_connection (nm->priv->connection), 
-		        message, 
-			NULL);
+    if (message != NULL)
+    {
+      dbus_connection_send (dbus_g_connection_get_connection (nm->priv->connection), 
+      		            message, 
+      		            NULL);
+    }
   
-  dbus_message_unref (message);
+    g_free (hint);
+  }
+
+  if (dest != NULL)
+  {
+    message = hildon_desktop_notification_manager_create_signal (nm, 
+		    						 id, 
+								 dest, 
+								 "ActionInvoked");
+
+    g_assert (message != NULL);
+
+    dbus_message_append_args (message,
+                              DBUS_TYPE_STRING, &action_id,
+                              DBUS_TYPE_INVALID);
+
+    dbus_connection_send (dbus_g_connection_get_connection (nm->priv->connection), 
+          	          message, 
+          		  NULL);
+    
+    dbus_message_unref (message);
+    
+    g_free (dest);
+  }
 }
