@@ -73,6 +73,9 @@ struct _HildonDesktopNotificationManager
 struct _HildonDesktopNotificationManagerClass 
 {
   GtkListStoreClass parent_class;
+
+  void     *(*notification_closed)    (HildonDesktopNotificationManager *nm,
+		                       gint id);
 };
 
 GType      hildon_desktop_notification_manager_get_type           (void);
@@ -90,6 +93,15 @@ gboolean   hildon_desktop_notification_manager_notify             (HildonDesktop
                                                       		   gint                   timeout, 
                                                       		   DBusGMethodInvocation *context);
 
+gboolean   hildon_desktop_notification_manager_system_note_infoprint  (HildonDesktopNotificationManager *nm,
+								       const gchar           *message,
+								       DBusGMethodInvocation *context);
+
+gboolean   hildon_desktop_notification_manager_system_note_dialog (HildonDesktopNotificationManager *nm,
+								   const gchar           *message,
+								   guint                  type,
+								   DBusGMethodInvocation *context);
+
 gboolean   hildon_desktop_notification_manager_get_capabilities   (HildonDesktopNotificationManager *nm, 
                                                       		   gchar               ***caps);
 
@@ -103,6 +115,10 @@ gboolean   hildon_desktop_notification_manager_close_notification (HildonDesktop
                                                                    guint id, 
 								   GError **error);
 
+gboolean   hildon_desktop_notification_manager_find_by_id         (HildonDesktopNotificationManager *nm,
+		 						   guint id,
+								   GtkTreeIter *return_iter);
+
 void       hildon_desktop_notification_manager_call_action        (HildonDesktopNotificationManager *nm,
                                                                    guint                  id,
 								   const gchar           *action_id);
