@@ -39,6 +39,7 @@
 #include <libhildondesktop/hildon-desktop-home-item.h>
 
 #include <gtk/gtkcheckmenuitem.h>
+#include <gtk/gtkspinbutton.h>
 
 #ifdef HAVE_LIBHILDON
 #include <hildon/hildon-banner.h>
@@ -62,8 +63,11 @@
 
 #define HD_HOME_BACKGROUND_CONF_FILE        "home-background.conf"
 #define HD_DESKTOP_USER_PATH                ".osso/hildon-desktop/"
-#define HD_HOME_BACKGROUND_GLOBAL_CONF_FILE \
-    HD_DESKTOP_CONFIG_PATH G_DIR_SEPARATOR_S HD_HOME_BACKGROUND_CONF_FILE
+#define HD_HOME_BACKGROUND_DEFAULT          HD_DESKTOP_BACKGROUNDS_PATH\
+                                            "/default.desktop"
+
+#define BG_DESKTOP_GROUP           "Desktop Entry"
+#define BG_DESKTOP_IMAGE_FILENAME  "File"
 
 #define LAYOUT_OPENING_BANNER_TIMEOUT   2500
 
@@ -207,6 +211,11 @@ hd_home_window_show_information_banner (HDHomeWindow *window,
                                         const gchar *text);
 #endif
 
+#if 0
+static void
+hd_home_window_adjust_alpha_activate (HDHomeWindow *window);
+#endif
+
 static void
 hd_home_window_class_init (HDHomeWindowClass *window_class)
 {
@@ -251,7 +260,7 @@ hd_home_window_class_init (HDHomeWindowClass *window_class)
   pspec = g_param_spec_boolean ("layout-mode-sucks",
                                 "Layout mode sucks",
                                 "Whether or not the layout mode sucks",
-                                FALSE,
+                                TRUE,
                                (G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
   g_object_class_install_property (object_class,
                                    PROP_LAYOUT_MODE_SUCKS,
@@ -337,20 +346,46 @@ hd_home_window_constructor (GType                   gtype,
 
   if (error)
     {
+      GKeyFile         *keyfile;
+      const GdkColor    default_color = {0, 0, 0, 0};
+      gchar            *filename = NULL;
+
       g_clear_error (&error);
 
-      /* Try the global configuration file */
-      hd_home_background_load (background,
-                               HD_HOME_BACKGROUND_GLOBAL_CONF_FILE,
-                               &error);
+      /* Revert to the default background file */
+      keyfile = g_key_file_new ();
+      g_key_file_load_from_file (keyfile,
+                                 HD_HOME_BACKGROUND_DEFAULT,
+                                 G_KEY_FILE_NONE,
+                                 &error);
+      if (error) goto error;
 
+      filename = g_key_file_get_string (keyfile,
+                                        BG_DESKTOP_GROUP,
+                                        BG_DESKTOP_IMAGE_FILENAME,
+                                        &error);
+
+      if (error) goto error;
+
+      background = g_object_new (HD_TYPE_HOME_BACKGROUND,
+                                 "filename", filename,
+                                 "color", &default_color,
+                                 NULL);
+
+error:
       if (error)
         {
-          g_warning ("Error when loading background parameters from: %s",
+          g_warning ("Could not load default background: %s",
                      error->message);
-          g_error_free (error);
-          background = NULL;
+          g_clear_error (&error);
+          background = g_object_new (HD_TYPE_HOME_BACKGROUND,
+                                     "filename", HD_HOME_BACKGROUND_NO_IMAGE,
+                                     "color", &default_color,
+                                     NULL);
         }
+
+      g_free (filename);
+
     }
 
 
@@ -664,7 +699,6 @@ hd_home_window_build_main_menu (HDHomeWindow *window)
 {
   HDHomeWindowPrivate  *priv;
   GtkWidget            *menu;
-  GtkWidget            *tools_menu;
   GtkWidget            *settings_menu;
   GtkWidget            *menu_item;
 
@@ -698,6 +732,42 @@ hd_home_window_build_main_menu (HDHomeWindow *window)
   priv->settings_item = menu_item;
   priv->settings_menu = settings_menu;
 
+  /* set background */
+  menu_item = gtk_menu_item_new_with_label (HH_MENU_SET_BACKGROUND);
+  g_signal_connect_swapped (menu_item, "activate",
+                            G_CALLBACK (hd_home_window_set_background_activate),
+                            window);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+  gtk_widget_show (menu_item);
+
+#ifdef HAVE_LIBOSSO
+  /* personalisation */
+  menu_item = gtk_menu_item_new_with_label (HH_MENU_PERSONALISATION);
+  g_signal_connect_swapped (menu_item, "activate",
+                            G_CALLBACK (hd_home_window_personalisation_activate),
+                            window);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+  gtk_widget_show (menu_item);
+
+  /* calibration */
+  menu_item = gtk_menu_item_new_with_label (HH_MENU_CALIBRATION);
+  g_signal_connect_swapped (menu_item, "activate",
+                            G_CALLBACK (hd_home_window_calibration_activate),
+                            window);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+  gtk_widget_show (menu_item);
+
+#ifdef HAVE_LIBHILDONHELP
+  /* help */
+  menu_item = gtk_menu_item_new_with_label (HH_MENU_HELP);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+  g_signal_connect_swapped (menu_item, "activate",
+                            G_CALLBACK (hd_home_window_help_activate),
+                            window);
+  gtk_widget_show (menu_item);
+#endif
+#endif
+
   if (!priv->layout_mode_sucks)
     {
       /* layout mode */
@@ -718,6 +788,7 @@ hd_home_window_build_main_menu (HDHomeWindow *window)
       priv->layout_mode_item = menu_item;
     }
 
+#if 0
   else
     {
       GtkWidget *area;
@@ -737,50 +808,16 @@ hd_home_window_build_main_menu (HDHomeWindow *window)
         }
       gtk_widget_show (menu_item);
     }
+
   
-  /* tools sub-menu */
-  tools_menu = gtk_menu_new ();
-  menu_item = gtk_menu_item_new_with_label (HH_MENU_TOOLS);
-  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), tools_menu);
+  menu_item = gtk_menu_item_new_with_label ("Adjust transparency");
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-  gtk_widget_show (menu_item);
-
-  /* set background */
-  menu_item = gtk_menu_item_new_with_label (HH_MENU_SET_BACKGROUND);
   g_signal_connect_swapped (menu_item, "activate",
-                            G_CALLBACK (hd_home_window_set_background_activate),
-                            window);
-  gtk_menu_shell_append (GTK_MENU_SHELL (tools_menu), menu_item);
-  gtk_widget_show (menu_item);
-
-#ifdef HAVE_LIBOSSO
-  /* personalisation */
-  menu_item = gtk_menu_item_new_with_label (HH_MENU_PERSONALISATION);
-  g_signal_connect_swapped (menu_item, "activate",
-                            G_CALLBACK (hd_home_window_personalisation_activate),
-                            window);
-  gtk_menu_shell_append (GTK_MENU_SHELL (tools_menu), menu_item);
-  gtk_widget_show (menu_item);
-
-  /* calibration */
-  menu_item = gtk_menu_item_new_with_label (HH_MENU_CALIBRATION);
-  g_signal_connect_swapped (menu_item, "activate",
-                            G_CALLBACK (hd_home_window_calibration_activate),
-                            window);
-  gtk_menu_shell_append (GTK_MENU_SHELL (tools_menu), menu_item);
-  gtk_widget_show (menu_item);
-
-#ifdef HAVE_LIBHILDONHELP
-  /* help */
-  menu_item = gtk_menu_item_new_with_label (HH_MENU_HELP);
-  gtk_menu_shell_append (GTK_MENU_SHELL (tools_menu), menu_item);
-  g_signal_connect_swapped (menu_item, "activate",
-                            G_CALLBACK (hd_home_window_help_activate),
+                            G_CALLBACK (hd_home_window_adjust_alpha_activate),
                             window);
   gtk_widget_show (menu_item);
 #endif
-#endif
-  
+
   return menu;
 
 }
@@ -1225,6 +1262,7 @@ hd_home_window_set_background_reponse (HDHomeWindow *window,
     {
       case GTK_RESPONSE_OK:
           gtk_widget_hide (GTK_WIDGET(dialog));
+          g_debug ("BACKGROUND_OK");
           if (!hd_home_background_equal (priv->background,
                                          background))
             hd_home_background_apply_async 
@@ -1275,8 +1313,8 @@ hd_home_window_set_background_activate (HDHomeWindow *window)
 #ifdef HAVE_LIBOSSO
                          "osso-context", priv->osso_context,
 #endif
-                         "background-dir", HD_DESKTOP_BACKGROUNDS_PATH,
                          "background", priv->background,
+                         "background-dir", HD_DESKTOP_BACKGROUNDS_PATH,
                          NULL);
 
   g_signal_connect_swapped (dialog, "response",
@@ -1312,3 +1350,58 @@ hd_home_window_show_information_banner (HDHomeWindow *window,
                                   text);
 }
 #endif
+
+
+#if 0
+static void
+scale_changed (GtkRange *range, GtkWidget *area)
+{
+  gint  alpha;
+  
+  alpha = (gint) gtk_range_get_value (range);
+
+  g_debug ("value_changed");
+
+  g_object_set (area,
+                "applet-default-alpha", alpha,
+                NULL);
+}
+
+static void
+hd_home_window_adjust_alpha_activate (HDHomeWindow *window)
+{
+  GtkWidget    *area, *dialog, *scale;
+  gint          alpha;
+  gint          response;
+
+  area = hildon_home_window_get_area (HILDON_HOME_WINDOW (window));
+
+  g_object_get (area,
+                "applet-default-alpha", &alpha,
+                NULL);
+
+  dialog = gtk_dialog_new_with_buttons ("Adjust applets' transparency",
+                                        GTK_WINDOW (window),
+                                        GTK_DIALOG_MODAL | 
+                                          GTK_DIALOG_DESTROY_WITH_PARENT,
+                                        GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
+                                        NULL);
+  gtk_widget_set_size_request (dialog, 300, -1);
+
+  scale = gtk_hscale_new_with_range (0.0,
+                                     100.0,
+                                     1.0);
+  gtk_widget_show (scale);
+  g_signal_connect (scale, "value-changed",
+                    G_CALLBACK (scale_changed),
+                    area);
+  gtk_range_set_value (GTK_RANGE (scale), (gdouble)alpha);
+  
+  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), scale);
+
+  response = gtk_dialog_run (GTK_DIALOG (dialog));
+  
+  gtk_widget_destroy (dialog);
+}
+
+#endif
