@@ -81,7 +81,7 @@ G_DEFINE_TYPE (HNOthersButton, hn_others_button, TASKNAVIGATOR_TYPE_ITEM);
 #define MENU_ITEM_EMPTY_SUBMENU_STRING _( "tana_li_of_noapps" )
 
 #define BORDER_WIDTH 7
-#define BUTTON_HEIGHT 90
+#define BUTTON_HEIGHT 80
 
 #define MENU_Y_POS 180
 #define MENU_MAX_WIDTH 360
@@ -176,7 +176,7 @@ hn_others_button_init (HNOthersButton *button)
   gtk_widget_show_all (priv->button);
 
   hdwm = hd_wm_get_singleton ();
-  hd_wm_set_others_menu_button (hdwm, priv->button);
+  hd_wm_set_all_menu_button (hdwm, priv->button);
 
   g_signal_connect (G_OBJECT (priv->button), 
                     "button-press-event",
@@ -686,11 +686,6 @@ hn_others_button_create_menu (HNOthersButton *button)
 		    G_CALLBACK (hn_others_menu_key_press),
 		    button);
     
-  g_signal_connect (G_OBJECT (menu), 
-		    "button-release-event",
-		    G_CALLBACK (hn_app_switcher_menu_button_release_cb),
-		    NULL);
-
   if (TASKNAVIGATOR_ITEM (button)->menu)
   {
     /* Destroy the previous version of the menu */
